# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.2.1

import enum

import winrt.system
from winrt._winrt_windows_devices_display import (
    DisplayMonitor,
)

__all__ = [
    "DisplayMonitorConnectionKind",
    "DisplayMonitorDescriptorKind",
    "DisplayMonitorPhysicalConnectorKind",
    "DisplayMonitorUsageKind",
    "DisplayMonitor",
]

class DisplayMonitorConnectionKind(enum.IntEnum):
    INTERNAL = 0
    WIRED = 1
    WIRELESS = 2
    VIRTUAL = 3

class DisplayMonitorDescriptorKind(enum.IntEnum):
    EDID = 0
    DISPLAY_ID = 1

class DisplayMonitorPhysicalConnectorKind(enum.IntEnum):
    UNKNOWN = 0
    HD15 = 1
    ANALOG_T_V = 2
    DVI = 3
    HDMI = 4
    LVDS = 5
    SDI = 6
    DISPLAY_PORT = 7

class DisplayMonitorUsageKind(enum.IntEnum):
    STANDARD = 0
    HEAD_MOUNTED = 1
    SPECIAL_PURPOSE = 2

